@@ -27,7 +27,10 @@ __license__ = "Apache 2.0"
 
 import kineticstoolkit.config as config
 import os
-from kineticstoolkit.decorators import unstable
+
+
+def __dir__():
+    return ["download"]
 
 
 def download(filename: str, **kwargs) -> str:
@@ -35,12 +38,9 @@ def download(filename: str, **kwargs) -> str:
     Download example data and return its local file name.
 
     This function download example data from github. These data are the same
-    that are used to generate the
-    [documentation website](https://kineticstoolkit.uqam.ca).
+    that are used to generate the documentation website:
 
-    These example data are volatile; they are supplied only for the user to
-    reproduce the tutorials. Therefore, these data may change according to
-    changes in the tutorials.
+    https://kineticstoolkit.uqam.ca
 
     Parameters
     ----------
@@ -52,6 +52,16 @@ def download(filename: str, **kwargs) -> str:
     str
         The file name (with complete path) of the downloaded sample data.
 
+    Caution
+    -------
+    These example data are volatile; they are supplied only for the user to
+    reproduce the tutorials. Therefore, these data may change according to
+    changes in the tutorials.
+
+    Note
+    ----
+    The ``**kwargs`` arguments are for debugging and internal use only.
+
     """
     # Additional information for developers:
     # kwargs may include force_download=True, to force download from github.
