@@ -30,12 +30,15 @@ import scipy.signal as sgl
 import scipy.ndimage as ndi
 import warnings
 from kineticstoolkit import TimeSeries
-from kineticstoolkit.decorators import directory
 from typing import Tuple, Union, Sequence, List
 
 import kineticstoolkit as ktk  # for doctests
 
 
+def __dir__():
+    return ["savgol", "smooth", "butter", "deriv", "median"]
+
+
 def _interpolate(ts: TimeSeries, key: str) -> Tuple[TimeSeries, np.ndarray]:
     """Interpolate NaNs in a given data key in a TimeSeries."""
     ts = ts.get_subset(key)
@@ -44,25 +47,41 @@ def _interpolate(ts: TimeSeries, key: str) -> Tuple[TimeSeries, np.ndarray]:
     if not np.all(~nan_index):
         # There were NaNs, issue a warning.
         ts = ts.fill_missing_samples(0)
-        warnings.warn('NaNs found in the signal. They have been '
-                      'interpolated before filtering, and then put '
-                      'back in the filtered data.')
+        warnings.warn(
+            "NaNs found in the signal. They have been "
+            "interpolated before filtering, and then put "
+            "back in the filtered data."
+        )
     return (ts, nan_index)
 
 
-def savgol(ts: TimeSeries, /, *, window_length: int, poly_order: int,
-           deriv: int = 0) -> TimeSeries:
+def _validate_input(ts):
+    """
+    Check that time is not null, that sample rate is constant, and that
+    time unit is s.
+    """
+    if ts.time.shape[0] == 0:
+        raise ValueError("There is no data to filter.")
+    if np.isnan(ts.get_sample_rate()):
+        raise ValueError("Sample rate must be constant.")
+    try:
+        assert ts.time_info["Unit"] == "s"
+    except KeyError:
+        warnings.warn("I assume, but cannot tell if the time unit is 's'.")
+    except ValueError:
+        warnings.warn("It seems that unit is not 's'.")
+
+
+def savgol(
+    ts: TimeSeries, /, *, window_length: int, poly_order: int, deriv: int = 0
+) -> TimeSeries:
     """
     Apply a Savitzky-Golay filter on a TimeSeries.
 
-    On n-dimensional data, filtering occurs on the first axis (time).
-
-    Notes
-    -----
-    - The sampling rate must be constant.
-    - If the TimeSeries contains missing samples, a warning is issued, missing
-      samples are interpolated using a first-order interpolation before
-      filtering, and then replaced by NaNs in the filtered signal.
+    Filtering occurs on the first axis (time). If the TimeSeries contains
+    missing samples, a warning is issued, missing samples are interpolated
+    using a first-order interpolation before filtering, and then replaced by
+    np.nan in the filtered signal.
 
     Parameters
     ----------
@@ -78,7 +97,25 @@ def savgol(ts: TimeSeries, /, *, window_length: int, poly_order: int,
         Optional. The order of the derivative to compute. The default is 0,
         which means to filter the data without differentiating.
 
+    Returns
+    -------
+    TimeSeries
+        A copy of the input TimeSeries, which each data being filtered.
+
+    Raises
+    ------
+    ValueError
+        If sample rate is not constant, or if there is no data to
+        filter.
+
+    See also
+    --------
+    ktk.filters.smooth
+    ktk.filters.median
+
     """
+    _validate_input(ts)
+
     tsout = ts.copy()
 
     delta = ts.time[1] - ts.time[0]
@@ -89,16 +126,15 @@ def savgol(ts: TimeSeries, /, *, window_length: int, poly_order: int,
 
         if np.sum(~nan_index) < poly_order + 1:
             # We can't do anything without more points
-            warnings.warn(
-                f"Not enough non-missing samples to filter {key}.")
+            warnings.warn(f"Not enough non-missing samples to filter {key}.")
             continue
 
         input_signal = subts.data[key]
 
         # Filter
-        filtered_data = sgl.savgol_filter(input_signal,
-                                          window_length, poly_order, deriv,
-                                          delta=delta, axis=0)
+        filtered_data = sgl.savgol_filter(
+            input_signal, window_length, poly_order, deriv, delta=delta, axis=0
+        )
 
         # Put back NaNs
         filtered_data[nan_index] = np.nan
@@ -113,14 +149,10 @@ def smooth(ts: TimeSeries, /, window_length: int) -> TimeSeries:
     """
     Apply a smoothing (moving average) filter on a TimeSeries.
 
-    On n-dimensional data, filtering occurs on the first axis (time).
-
-    Notes
-    -----
-    - The sampling rate must be constant.
-    - If the TimeSeries contains missing samples, a warning is issued, missing
-      samples are interpolated using a first-order interpolation before
-      filtering, and then replaced by NaNs in the filtered signal.
+    Filtering occurs on the first axis (time). If the TimeSeries contains
+    missing samples, a warning is issued, missing samples are interpolated
+    using a first-order interpolation before filtering, and then replaced by
+    np.nan in the filtered signal.
 
     Parameters
     ----------
@@ -130,24 +162,45 @@ def smooth(ts: TimeSeries, /, window_length: int) -> TimeSeries:
         The length of the filter window. window_length must be a positive
         odd integer less or equal than the length of the TimeSeries.
 
+    Returns
+    -------
+    TimeSeries
+        A copy of the input TimeSeries, which each data being filtered.
+
+    Raises
+    ------
+    ValueError
+        If sample rate is not constant, or if there is no data to
+        filter.
+
+    See also
+    --------
+    ktk.filters.savgol
+    ktk.filters.median
+
     """
+    _validate_input(ts)
+
     tsout = savgol(ts, window_length=window_length, poly_order=0)
     return tsout
 
 
-def butter(ts: TimeSeries, /, fc: Union[float, Sequence], *, order: int = 2,
-           btype: str = 'lowpass', filtfilt: bool = True) -> TimeSeries:
+def butter(
+    ts: TimeSeries,
+    /,
+    fc: Union[float, Sequence],
+    *,
+    order: int = 2,
+    btype: str = "lowpass",
+    filtfilt: bool = True,
+) -> TimeSeries:
     """
     Apply a Butterworth filter to a TimeSeries.
 
-    On n-dimensional data, filtering occurs on the first axis (time).
-
-    Note
-    ----
-    - The sampling rate must be constant.
-    - If the TimeSeries contains missing samples, a warning is issued, missing
-      samples are interpolated using a first-order interpolation before
-      filtering, and then replaced by NaNs in the filtered signal.
+    Filtering occurs on the first axis (time). If the TimeSeries contains
+    missing samples, a warning is issued, missing samples are interpolated
+    using a first-order interpolation before filtering, and then replaced by
+    np.nan in the filtered signal.
 
     Parameters
     ----------
@@ -158,24 +211,37 @@ def butter(ts: TimeSeries, /, fc: Union[float, Sequence], *, order: int = 2,
         (lowpass, highpass), or a sequence of two floats (e.g., [10., 13.])
         for two-frequency filters (bandpass, bandstop).
     order
-        Optional. Order of the filter.
+        Optional. Order of the filter. Default is 2.
     btype
-        Optional. {'lowpass', 'highpass', 'bandpass', 'bandstop'}.
+        Optional. Can be either 'lowpass', 'highpass', 'bandpass' or
+        'bandstop'. Default is 'lowpass'.
     filtfilt
         Optional. If True, the filter is applied two times in reverse direction
         to eliminate time lag. If False, the filter is applied only in forward
-        direction.
+        direction. Default is True.
+
+    Returns
+    -------
+    TimeSeries
+        A copy of the input TimeSeries, which each data being filtered.
+
+    Raises
+    ------
+    ValueError
+        If sample rate is not constant, or if there is no data to
+        filter.
 
     """
+    _validate_input(ts)
+
     ts = ts.copy()
 
     # Create the filter
-    fs = (1 / (ts.time[1] - ts.time[0]))
+    fs = 1 / (ts.time[1] - ts.time[0])
     if np.isnan(fs):
         raise ValueError("The TimeSeries' time vector must not contain NaNs.")
 
-    sos = sgl.butter(order, fc, btype, analog=False,
-                     output='sos', fs=fs)
+    sos = sgl.butter(order, fc, btype, analog=False, output="sos", fs=fs)
 
     for data in ts.data:
 
@@ -183,11 +249,9 @@ def butter(ts: TimeSeries, /, fc: Union[float, Sequence], *, order: int = 2,
 
         # Filter
         if filtfilt is True:
-            subts.data[data] = sgl.sosfiltfilt(sos, subts.data[data],
-                                               axis=0)
+            subts.data[data] = sgl.sosfiltfilt(sos, subts.data[data], axis=0)
         else:
-            subts.data[data] = sgl.sosfilt(sos, subts.data[data],
-                                           axis=0)
+            subts.data[data] = sgl.sosfilt(sos, subts.data[data], axis=0)
 
         # Put back nans
         subts.data[data][missing] = np.nan
@@ -202,11 +266,8 @@ def deriv(ts: TimeSeries, /, n: int = 1) -> TimeSeries:
     """
     Calculate the nth numerical derivative.
 
-    On n-dimensional data, filtering occurs on the first axis (time).
-
-    Notes
-    -----
-    - The sample rate must be constant.
+    Filtering occurs on the first axis (time). The sample rate must be
+    constant.
 
     Parameters
     ----------
@@ -219,8 +280,14 @@ def deriv(ts: TimeSeries, /, n: int = 1) -> TimeSeries:
     Returns
     -------
     TimeSeries
-        A copy of the input TimeSeries, with n less points than the original
-        one.
+        A copy of the input TimeSeries, which each data being derived. The
+        length of the resulting TimeSeries is one less than `ts`.
+
+    Raises
+    ------
+    ValueError
+        If sample rate is not constant, or if there is no data to
+        filter.
 
     Example
     -------
@@ -250,14 +317,17 @@ def deriv(ts: TimeSeries, /, n: int = 1) -> TimeSeries:
     array([ 100., -100., -100.])
 
     """
+    _validate_input(ts)
+
     out_ts = ts.copy()
 
     for i in range(n):
         out_ts.time = (out_ts.time[1:] + out_ts.time[0:-1]) / 2
 
     for key in ts.data:
-        out_ts.data[key] = np.diff(
-            ts.data[key], n=n, axis=0) / (ts.time[1] - ts.time[0]) ** n
+        out_ts.data[key] = (
+            np.diff(ts.data[key], n=n, axis=0) / (ts.time[1] - ts.time[0]) ** n
+        )
 
     return out_ts
 
@@ -266,7 +336,7 @@ def median(ts: TimeSeries, /, window_length: int = 3) -> TimeSeries:
     """
     Calculate a moving median.
 
-    On n-dimensional data, filtering occurs on the first axis (time).
+    Filtering occurs on the first axis (time).
 
     Parameters
     ----------
@@ -276,6 +346,11 @@ def median(ts: TimeSeries, /, window_length: int = 3) -> TimeSeries:
     window_length
         Optional. Kernel size, must be odd. The default is 3.
 
+    See also
+    --------
+    ktk.butter.smooth
+    ktk.butter.savgol
+
     Example
     -------
     >>> ts = ktk.TimeSeries(time=np.arange(0, 0.5, 0.1))
@@ -289,19 +364,12 @@ def median(ts: TimeSeries, /, window_length: int = 3) -> TimeSeries:
     for key in ts.data:
         window_shape = [1 for i in range(len(ts.data[key].shape))]
         window_shape[0] = window_length
-        out_ts.data[key] = ndi.median_filter(
-            ts.data[key], size=window_shape)
+        out_ts.data[key] = ndi.median_filter(ts.data[key], size=window_shape)
 
     return out_ts
 
 
-module_locals = locals()
-
-
-def __dir__():
-    return directory(module_locals)
-
-
 if __name__ == "__main__":
     import doctest
+
     doctest.testmod(optionflags=doctest.NORMALIZE_WHITESPACE)
