@@ -26,9 +26,8 @@ __license__ = "Apache 2.0"
 
 
 import kineticstoolkit.geometry as geometry
-from kineticstoolkit import TimeSeries
-from kineticstoolkit.decorators import unstable, directory, deprecated
-from os.path import exists
+from kineticstoolkit import TimeSeries, read_c3d, write_c3d
+from kineticstoolkit.decorators import deprecated
 from typing import Sequence, Dict, Union
 from copy import deepcopy
 
@@ -37,219 +36,12 @@ import warnings
 import struct  # To unpack data from N3D files
 
 
-def read_c3d_file(filename: str) -> TimeSeries:
-    """
-    Read markers from a C3D file.
-
-    The markers positions are returned in a TimeSeries where each marker
-    corresponds to a data key. Each marker position is expressed in this form:
-
-    array([[x0, y0, z0, 1.], [x1, y1, z1, 1.], [x2, y2, z2, 1.], ...])
-
-    Parameters
-    ----------
-    filename
-        Path of the C3D file
-
-    Notes
-    -----
-    - This function relies on `ezc3d`, which is available on
-      conda-forge and on git-hub. Please install ezc3d before using
-      read_c3d_file. https://github.com/pyomeca/ezc3d
-
-    - The point unit must be either mm or m. In both cases, the final unit
-      returned in the output TimeSeries is m.
-
-    - As for any instrument, please check that your data loads correctly on
-      your first use (e.g., sampling frequency, position unit). It is
-      possible that read_c3d_file misses some corner cases.
-
-    """
-    try:
-        import ezc3d
-    except ModuleNotFoundError:
-        raise ModuleNotFoundError(
-            "The optional module ezc3d is not installed, but it is required "
-            "to use this function. Please install it using: "
-            "conda install -c conda-forge ezc3d"
-        )
-
-    # Create the reader
-    if isinstance(filename, str) and exists(filename):
-        reader = ezc3d.c3d(filename)
-    else:
-        raise FileNotFoundError(f"File {filename} was not found.")
-
-    # Create the output timeseries
-    output = TimeSeries()
-
-    # Get the marker label names and create a timeseries data entry for each
-    # Get the labels
-    labels = reader["parameters"]["POINT"]["LABELS"]["value"]
-    n_frames = reader["parameters"]["POINT"]["FRAMES"]["value"][0]
-    point_rate = reader["parameters"]["POINT"]["RATE"]["value"][0]
-    point_unit = reader["parameters"]["POINT"]["UNITS"]["value"][0]
-
-    if point_unit == "mm":
-        point_factor = 0.001
-    elif point_unit == "m":
-        point_factor = 1
-    else:
-        raise (ValueError("Point unit must be 'm' or 'mm'."))
-
-    n_labels = len(labels)
-    for i_label in range(n_labels):
-        # Strip leading and ending spaces
-        labels[i_label] = labels[i_label].strip()
-
-        label_name = labels[i_label]
-
-        output.data[label_name] = np.array(
-            [point_factor, point_factor, point_factor, 1]
-            * reader["data"]["points"][:, i_label, :].T
-        )
-
-        output = output.add_data_info(label_name, "Unit", "m")
-
-    # Create the timeseries time vector
-    output.time = np.arange(n_frames) / point_rate
-
-    return output
-
-
-def write_c3d_file(filename: str, markers: TimeSeries) -> None:
-    """
-    Write a markers TimeSeries to a C3D file.
-
-    Parameters
-    ----------
-    filename
-        Path of the C3D file
-
-    markers
-        Markers trajectories, following the same form as the TimeSeries
-        read by read_c3d_file.
-
-    Notes
-    -----
-    This function relies on `ezc3d`. Please install ezc3d before using
-    write_c3d_file. https://github.com/pyomeca/ezc3d
-
-    """
-    try:
-        import ezc3d
-    except ModuleNotFoundError:
-        raise ModuleNotFoundError(
-            "The optional module ezc3d is not installed, but it is required "
-            "to use this function. Please install it using: "
-            "conda install -c conda-forge ezc3d"
-        )
-
-    sample_rate = (markers.time.shape[0] - 1) / (
-        markers.time[-1] - markers.time[0]
-    )
-
-    marker_list = []
-    marker_data = np.zeros((4, len(markers.data), len(markers.time)))
-
-    for i_marker, marker in enumerate(markers.data):
-        marker_list.append(marker)
-        marker_data[0, i_marker, :] = markers.data[marker][:, 0]
-        marker_data[1, i_marker, :] = markers.data[marker][:, 1]
-        marker_data[2, i_marker, :] = markers.data[marker][:, 2]
-        marker_data[3, i_marker, :] = markers.data[marker][:, 3]
-
-    # Load an empty c3d structure
-    c3d = ezc3d.c3d()
-
-    # Fill it with data
-    c3d["parameters"]["POINT"]["RATE"]["value"] = [sample_rate]
-    c3d["parameters"]["POINT"]["LABELS"]["value"] = tuple(marker_list)
-    c3d["data"]["points"] = marker_data
-
-    # Add a custom parameter to the POINT group
-    c3d.add_parameter("POINT", "UNITS", "m")
-
-    # Write the data
-    c3d.write(filename)
-
-
-def read_n3d_file(filename: str, labels: Sequence[str] = []) -> TimeSeries:
-    """
-    Read markers from an NDI N3D file.
-
-    The markers positions are returned in a TimeSeries where each marker
-    corresponds to a data key. Each marker position is expressed in this form:
-
-    array([[x0, y0, z0, 1.], [x1, y1, z1, 1.], [x2, y2, z2, 1.], ...])
-
-    Parameters
-    ----------
-    filename : str
-        Path of the N3D file.
-    labels : list of str (optional)
-        Marker names
-
-    Returns
-    -------
-    TimeSeries
-
-    """
-    with open(filename, "rb") as fid:
-        _ = fid.read(1)  # 32
-        n_markers = struct.unpack("h", fid.read(2))[0]
-        n_data_per_marker = struct.unpack("h", fid.read(2))[0]
-        n_columns = n_markers * n_data_per_marker
-
-        n_frames = struct.unpack("i", fid.read(4))[0]
-
-        collection_frame_frequency = struct.unpack("f", fid.read(4))[0]
-        user_comments = struct.unpack("60s", fid.read(60))[0]
-        system_comments = struct.unpack("60s", fid.read(60))[0]
-        file_description = struct.unpack("30s", fid.read(30))[0]
-        cutoff_filter_frequency = struct.unpack("h", fid.read(2))[0]
-        time_of_collection = struct.unpack("8s", fid.read(8))[0]
-        _ = fid.read(2)
-        date_of_collection = struct.unpack("8s", fid.read(8))[0]
-        extended_header = struct.unpack("73s", fid.read(73))[0]
-
-        # Read the rest and put it in an array
-        ndi_array = np.ones((n_frames, n_columns)) * np.NaN
-
-        for i_frame in range(n_frames):
-            for i_column in range(n_columns):
-                data = struct.unpack("f", fid.read(4))[0]
-                if data < -1e25:  # technically, it is -3.697314e+28
-                    data = np.NaN
-                ndi_array[i_frame, i_column] = data
-
-        # Conversion from mm to meters
-        ndi_array /= 1000
-
-        # Transformation to a TimeSeries
-        ts = TimeSeries(
-            time=np.linspace(
-                0, n_frames / collection_frame_frequency, n_frames
-            )
-        )
-
-        for i_marker in range(n_markers):
-            if labels != []:
-                label = labels[i_marker]
-            else:
-                label = f"Marker{i_marker}"
-
-            ts.data[label] = np.block(
-                [
-                    [
-                        ndi_array[:, 3 * i_marker : 3 * i_marker + 3],
-                        np.ones((n_frames, 1)),
-                    ]
-                ]
-            )
-            ts = ts.add_data_info(label, "Unit", "m")
-
-    return ts
+def __dir__():
+    return [
+        "create_cluster",
+        "extend_cluster",
+        "track_cluster",
+    ]
 
 
 # def read_xml_file(filename):
@@ -371,11 +163,6 @@ def create_cluster(
     """
     Create a cluster definition based on a static acquisition.
 
-    Warning
-    -------
-    This function, which has been introduced in 0.7, is still experimental and
-    may change signature or behaviour in the future.
-
     Parameters
     ----------
     markers
@@ -389,6 +176,16 @@ def create_cluster(
         Dictionary where each entry represents the local position of a marker
         in an arbitrary coordinate system.
 
+    Warning
+    -------
+    This function, which has been introduced in 0.7, is still experimental and
+    may change signature or behaviour in the future.
+
+    See also
+    --------
+    ktk.kinematics.extend_cluster
+    ktk.kinematics.track_cluster
+
     """
     n_samples = len(markers.time)
     n_markers = len(marker_names)
@@ -427,11 +224,6 @@ def extend_cluster(
     """
     Add a point to an existing cluster.
 
-    Warning
-    -------
-    This function, which has been introduced in 0.7, is still experimental and
-    may change signature or behaviour in the future.
-
     Parameters
     ----------
     markers
@@ -445,7 +237,17 @@ def extend_cluster(
     Returns
     -------
     Dict[str, np.ndarray]
-        The cluster with the added point.
+        A copy of the initial cluster, with the added point.
+
+    Warning
+    -------
+    This function, which has been introduced in 0.7, is still experimental and
+    may change signature or behaviour in the future.
+
+    See also
+    --------
+    ktk.kinematics.create_cluster
+    ktk.kinematics.track_cluster
 
     """
     cluster = deepcopy(cluster)
@@ -468,13 +270,8 @@ def track_cluster(
     """
     Fit a cluster to a TimeSeries of point trajectories.
 
-    Fits a cluster to a TimeSeries and reconstructs a solidified version of
-    all the points defined in this cluster.
-
-    Warning
-    -------
-    This function, which has been introduced in 0.7, is still experimental and
-    may change signature or behaviour in the future.
+    This function fits a cluster to a TimeSeries and reconstructs a solidified
+    version of all the points defined in this cluster.
 
     Parameters
     ----------
@@ -495,6 +292,16 @@ def track_cluster(
     TimeSeries
         A TimeSeries with the trajectories of all cluster points.
 
+    Warning
+    -------
+    This function, which has been introduced in 0.7, is still experimental and
+    may change signature or behaviour in the future.
+
+    See also
+    --------
+    ktk.kinematics.create_cluster
+    ktk.kinematics.track_cluster
+
     """
     out = markers.copy(copy_data=False, copy_data_info=False)
     unit = _get_marker_unit(markers)
@@ -563,7 +370,6 @@ def _get_marker_unit(markers: TimeSeries) -> Union[None, str]:
     return unit
 
 
-@unstable
 def write_trc_file(markers: TimeSeries, filename: str) -> None:
     """
     Export a markers TimeSeries to OpenSim's TRC file format.
@@ -576,6 +382,12 @@ def write_trc_file(markers: TimeSeries, filename: str) -> None:
     filename
         Name of the trc file to create.
 
+    Warning
+    -------
+    This function may eventually move either to the base namespace like
+    write_c3d_file, or to an opensim extension that is currently being
+    developed.
+
     """
     markers = markers.copy()
     markers.fill_missing_samples(0)
@@ -623,7 +435,6 @@ def write_trc_file(markers: TimeSeries, filename: str) -> None:
 
 
 # %% Deprecated
-@unstable
 @deprecated(
     since="master", until="January 2023", details="Use create_cluster()."
 )
@@ -652,7 +463,6 @@ def define_rigid_body(
     return create_cluster(kinematics, marker_names)
 
 
-@unstable
 @deprecated(
     since="master", until="January 2023", details="Use track_clusters()."
 )
@@ -705,7 +515,6 @@ def track_rigid_body(
     )
 
 
-@unstable
 @deprecated(
     since="master", until="January 2023", details="No replacement yet."
 )
@@ -751,7 +560,6 @@ def track_rigid_bodies(
     return out
 
 
-@unstable
 @deprecated(
     since="master", until="January 2023", details="Use extend_cluster()."
 )
@@ -811,11 +619,153 @@ def define_local_position(
     return local_points
 
 
-# %% Footer
+@deprecated(
+    since="0.9",
+    until="2024",
+    details=(
+        "Please use the ktk.read_c3d() function that is more powerful "
+        "since it can also read analog data."
+    ),
+)
+def read_c3d_file(filename: str) -> TimeSeries:
+    """
+    Read markers from a C3D file.
+
+    The markers positions are returned in a TimeSeries where each marker
+    corresponds to a data key. Each marker position is expressed in this form:
+
+    array([[x0, y0, z0, 1.], [x1, y1, z1, 1.], [x2, y2, z2, 1.], ...])
+
+    Parameters
+    ----------
+    filename
+        Path of the C3D file
+
+    Notes
+    -----
+    - This function relies on `ezc3d`, which is available on
+      conda-forge and on git-hub. Please install ezc3d before using
+      read_c3d_file. https://github.com/pyomeca/ezc3d
+
+    - The point unit must be either mm or m. In both cases, the final unit
+      returned in the output TimeSeries is m.
+
+    - As for any instrument, please check that your data loads correctly on
+      your first use (e.g., sampling frequency, position unit). It is
+      possible that read_c3d_file misses some corner cases.
+
+    """
+    return read_c3d(filename)["Points"]
 
 
-module_locals = locals()
+@deprecated(
+    since="0.9",
+    until="2024",
+    details=(
+        "Please use the ktk.write_c3d() function that is more powerful "
+        "since it can also write analog data."
+    ),
+)
+def write_c3d_file(filename: str, markers: TimeSeries) -> None:
+    """
+    Write a markers TimeSeries to a C3D file.
+
+    Parameters
+    ----------
+    filename
+        Path of the C3D file
+
+    markers
+        Markers trajectories, following the same form as the TimeSeries
+        read by read_c3d_file.
+
+    Notes
+    -----
+    This function relies on `ezc3d`. Please install ezc3d before using
+    write_c3d_file. https://github.com/pyomeca/ezc3d
+
+    """
+    write_c3d(filename, markers)
 
 
-def __dir__():
-    return directory(module_locals)
+@deprecated(
+    since="0.9",
+    until="2024",
+    details=("This function has been moved to the n3d extension."),
+)
+def read_n3d_file(filename: str, labels: Sequence[str] = []) -> TimeSeries:
+    """
+    Read markers from an NDI N3D file.
+
+    The markers positions are returned in a TimeSeries where each marker
+    corresponds to a data key. Each marker position is expressed in this form:
+
+    array([[x0, y0, z0, 1.], [x1, y1, z1, 1.], [x2, y2, z2, 1.], ...])
+
+    Parameters
+    ----------
+    filename : str
+        Path of the N3D file.
+    labels : list of str (optional)
+        Marker names
+
+    Returns
+    -------
+    TimeSeries
+
+    """
+    with open(filename, "rb") as fid:
+        _ = fid.read(1)  # 32
+        n_markers = struct.unpack("h", fid.read(2))[0]
+        n_data_per_marker = struct.unpack("h", fid.read(2))[0]
+        n_columns = n_markers * n_data_per_marker
+
+        n_frames = struct.unpack("i", fid.read(4))[0]
+
+        collection_frame_frequency = struct.unpack("f", fid.read(4))[0]
+        user_comments = struct.unpack("60s", fid.read(60))[0]
+        system_comments = struct.unpack("60s", fid.read(60))[0]
+        file_description = struct.unpack("30s", fid.read(30))[0]
+        cutoff_filter_frequency = struct.unpack("h", fid.read(2))[0]
+        time_of_collection = struct.unpack("8s", fid.read(8))[0]
+        _ = fid.read(2)
+        date_of_collection = struct.unpack("8s", fid.read(8))[0]
+        extended_header = struct.unpack("73s", fid.read(73))[0]
+
+        # Read the rest and put it in an array
+        ndi_array = np.ones((n_frames, n_columns)) * np.NaN
+
+        for i_frame in range(n_frames):
+            for i_column in range(n_columns):
+                data = struct.unpack("f", fid.read(4))[0]
+                if data < -1e25:  # technically, it is -3.697314e+28
+                    data = np.NaN
+                ndi_array[i_frame, i_column] = data
+
+        # Conversion from mm to meters
+        ndi_array /= 1000
+
+        # Transformation to a TimeSeries
+        ts = TimeSeries(
+            time=np.linspace(
+                0, n_frames / collection_frame_frequency, n_frames
+            )
+        )
+
+        for i_marker in range(n_markers):
+            if labels != []:
+                label = labels[i_marker]
+            else:
+                label = f"Marker{i_marker}"
+
+            ts.data[label] = np.block(
+                [
+                    [
+                        ndi_array[:, 3 * i_marker : 3 * i_marker + 3],
+                        np.ones((n_frames, 1)),
+                    ]
+                ]
+            )
+            ts = ts.add_data_info(label, "Unit", "m")
+
+    return ts
